@@ -42,6 +42,7 @@
 #include <sys/queue.h>
 #ifndef WIN32
 #include <sys/socket.h>
+#include <sys/wait.h>
 #include <sys/signal.h>
 #include <unistd.h>
 #include <netdb.h>
@@ -85,7 +86,7 @@ static struct event_base *global_base;
 #define read(fd,buf,len) recv((fd),(buf),(len),0)
 #endif
 
-void
+static void
 simple_read_cb(int fd, short event, void *arg)
 {
 	char buf[256];
@@ -104,7 +105,7 @@ simple_read_cb(int fd, short event, void *arg)
 	called++;
 }
 
-void
+static void
 simple_write_cb(int fd, short event, void *arg)
 {
 	int len;
@@ -116,7 +117,7 @@ simple_write_cb(int fd, short event, void *arg)
 		test_ok = 1;
 }
 
-void
+static void
 multiple_write_cb(int fd, short event, void *arg)
 {
 	struct event *ev = arg;
@@ -149,7 +150,7 @@ multiple_write_cb(int fd, short event, void *arg)
 	}
 }
 
-void
+static void
 multiple_read_cb(int fd, short event, void *arg)
 {
 	struct event *ev = arg;
@@ -171,7 +172,7 @@ multiple_read_cb(int fd, short event, void *arg)
 	}
 }
 
-void
+static void
 timeout_cb(int fd, short event, void *arg)
 {
 	struct timeval tv;
@@ -191,12 +192,13 @@ timeout_cb(int fd, short event, void *arg)
 		test_ok = 1;
 }
 
-void signal_cb_sa(int sig)
+static void
+signal_cb_sa(int sig)
 {
 	test_ok = 2;
 }
 
-void
+static void
 signal_cb(int fd, short event, void *arg)
 {
 	struct event *ev = arg;
@@ -210,7 +212,7 @@ struct both {
 	int nread;
 };
 
-void
+static void
 combined_read_cb(int fd, short event, void *arg)
 {
 	struct both *both = arg;
@@ -228,7 +230,7 @@ combined_read_cb(int fd, short event, void *arg)
 		exit(1);
 }
 
-void
+static void
 combined_write_cb(int fd, short event, void *arg)
 {
 	struct both *both = arg;
@@ -254,8 +256,8 @@ combined_write_cb(int fd, short event, void *arg)
 
 /* Test infrastructure */
 
-int
-setup_test(char *name)
+static int
+setup_test(const char *name)
 {
 
 	fprintf(stdout, "%s", name);
@@ -278,7 +280,7 @@ setup_test(char *name)
 	return (0);
 }
 
-int
+static int
 cleanup_test(void)
 {
 #ifndef WIN32
@@ -294,11 +296,11 @@ cleanup_test(void)
 		fprintf(stdout, "FAILED\n");
 		exit(1);
 	}
-
+        test_ok = 0;
 	return (0);
 }
 
-void
+static void
 test_simpleread(void)
 {
 	struct event ev;
@@ -317,7 +319,7 @@ test_simpleread(void)
 	cleanup_test();
 }
 
-void
+static void
 test_simplewrite(void)
 {
 	struct event ev;
@@ -333,7 +335,7 @@ test_simplewrite(void)
 	cleanup_test();
 }
 
-void
+static void
 test_multiple(void)
 {
 	struct event ev, ev2;
@@ -362,7 +364,7 @@ test_multiple(void)
 	cleanup_test();
 }
 
-void
+static void
 test_persistent(void)
 {
 	struct event ev, ev2;
@@ -391,7 +393,7 @@ test_persistent(void)
 	cleanup_test();
 }
 
-void
+static void
 test_combined(void)
 {
 	struct both r1, r2, w1, w2;
@@ -426,7 +428,7 @@ test_combined(void)
 	cleanup_test();
 }
 
-void
+static void
 test_simpletimeout(void)
 {
 	struct timeval tv;
@@ -446,7 +448,62 @@ test_simpletimeout(void)
 }
 
 #ifndef WIN32
-void
+extern struct event_base *current_base;
+static void
+test_fork(void)
+{
+	int status;
+	struct event ev;
+	pid_t pid;
+
+	setup_test("After fork: ");
+
+	write(pair[0], TEST1, strlen(TEST1)+1);
+
+	event_set(&ev, pair[1], EV_READ, simple_read_cb, &ev);
+	if (event_add(&ev, NULL) == -1)
+		exit(1);
+
+	if ((pid = fork()) == 0) {
+		/* in the child */
+		if (event_reinit(current_base) == -1) {
+			fprintf(stderr, "FAILED (reinit)\n");
+			exit(1);
+		}
+
+		event_dispatch();
+
+		/* we do not send an EOF; simple_read_cb requires an EOF 
+		 * to set test_ok.  we just verify that the callback was
+		 * called. */
+		exit(test_ok != 0 || called != 2);
+	}
+
+	/* wait for the child to read the data */
+	sleep(1);
+
+	write(pair[0], TEST1, strlen(TEST1)+1);
+
+	if (waitpid(pid, &status, 0) == -1) {
+		fprintf(stderr, "FAILED (fork)\n");
+		exit(1);
+	}
+	
+	if (WEXITSTATUS(status) != 0) {
+		fprintf(stderr, "FAILED (exit)\n");
+		exit(1);
+	}
+
+	/* test that the current event loop still works */
+	write(pair[0], TEST1, strlen(TEST1)+1);
+	shutdown(pair[0], SHUT_WR);
+
+	event_dispatch();
+
+	cleanup_test();
+}
+
+static void
 test_simplesignal(void)
 {
 	struct event ev;
@@ -469,11 +526,12 @@ test_simplesignal(void)
 	cleanup_test();
 }
 
-void
+static void
 test_immediatesignal(void)
 {
 	struct event ev;
 
+	test_ok = 0;
 	printf("Immediate signal: ");
 	signal_set(&ev, SIGUSR1, signal_cb, &ev);
 	signal_add(&ev, NULL);
@@ -483,7 +541,7 @@ test_immediatesignal(void)
 	cleanup_test();
 }
 
-void
+static void
 test_signal_dealloc(void)
 {
 	/* make sure that signal_event is event_del'ed and pipe closed */
@@ -494,15 +552,18 @@ test_signal_dealloc(void)
 	signal_add(&ev, NULL);
 	signal_del(&ev);
 	event_base_free(base);
+        /* If we got here without asserting, we're fine. */
+        test_ok = 1;
 	cleanup_test();
 }
 
-void
+static void
 test_signal_pipeloss(void)
 {
 	/* make sure that the base1 pipe is closed correctly. */
 	struct event_base *base1, *base2;
 	int pipe1;
+	test_ok = 0;
 	printf("Signal pipeloss: ");
 	base1 = event_init();
 	pipe1 = base1->sig.ev_signal_pair[0];
@@ -522,17 +583,19 @@ test_signal_pipeloss(void)
 /*
  * make two bases to catch signals, use both of them.  this only works
  * for event mechanisms that use our signal pipe trick.  kqueue handles
- * signals internally, and it looks like the first kqueue always gets the
- * signal.
+ * signals internally, and all interested kqueues get all the signals.
  */
-void
+static void
 test_signal_switchbase(void)
 {
 	struct event ev1, ev2;
 	struct event_base *base1, *base2;
+        int is_kqueue;
+	test_ok = 0;
 	printf("Signal switchbase: ");
 	base1 = event_init();
 	base2 = event_init();
+        is_kqueue = !strcmp(event_get_method(),"kqueue");
 	signal_set(&ev1, SIGUSR1, signal_cb, &ev1);
 	signal_set(&ev2, SIGUSR1, signal_cb, &ev2);
 	if (event_base_set(base1, &ev1) ||
@@ -547,15 +610,22 @@ test_signal_switchbase(void)
 	/* can handle signal before loop is called */
 	raise(SIGUSR1);
 	event_base_loop(base2, EVLOOP_NONBLOCK);
+        if (is_kqueue) {
+                if (!test_ok)
+                        goto done;
+                test_ok = 0;
+        }
 	event_base_loop(base1, EVLOOP_NONBLOCK);
-	if (test_ok) {
+	if (test_ok && !is_kqueue) {
 		test_ok = 0;
+
 		/* set base1 to handle signals */
 		event_base_loop(base1, EVLOOP_NONBLOCK);
 		raise(SIGUSR1);
 		event_base_loop(base1, EVLOOP_NONBLOCK);
 		event_base_loop(base2, EVLOOP_NONBLOCK);
 	}
+ done:
 	event_base_free(base1);
 	event_base_free(base2);
 	cleanup_test();
@@ -565,11 +635,12 @@ test_signal_switchbase(void)
  * assert that a signal event removed from the event queue really is
  * removed - with no possibility of it's parent handler being fired.
  */
-void
-test_signal_assert()
+static void
+test_signal_assert(void)
 {
 	struct event ev;
 	struct event_base *base = event_init();
+	test_ok = 0;
 	printf("Signal handler assert: ");
 	/* use SIGCONT so we don't kill ourselves when we signal to nowhere */
 	signal_set(&ev, SIGCONT, signal_cb, &ev);
@@ -595,8 +666,8 @@ test_signal_assert()
 /*
  * assert that we restore our previous signal handler properly.
  */
-void
-test_signal_restore()
+static void
+test_signal_restore(void)
 {
 	struct event ev;
 	struct event_base *base = event_init();
@@ -631,7 +702,7 @@ out:
 }
 #endif
 
-void
+static void
 test_free_active_base(void)
 {
 	struct event_base *base1;
@@ -647,7 +718,29 @@ test_free_active_base(void)
 	cleanup_test();
 }
 
-void
+static void
+test_event_base_new(void)
+{
+	struct event_base *base;
+	struct event ev1;
+	setup_test("Event base new: ");
+
+	write(pair[0], TEST1, strlen(TEST1)+1);
+	shutdown(pair[0], SHUT_WR);
+
+	base = event_base_new();
+	event_set(&ev1, pair[1], EV_READ, simple_read_cb, &ev1);
+	event_base_set(base, &ev1);
+	event_add(&ev1, NULL);
+
+	event_base_dispatch(base);
+
+	event_base_free(base);
+	test_ok = 1;
+	cleanup_test();
+}
+
+static void
 test_loopexit(void)
 {
 	struct timeval tv, tv_start, tv_end;
@@ -677,7 +770,43 @@ test_loopexit(void)
 	cleanup_test();
 }
 
-void
+static void
+break_cb(int fd, short events, void *arg)
+{
+	test_ok = 1;
+	event_loopbreak();
+}
+
+static void
+fail_cb(int fd, short events, void *arg)
+{
+	test_ok = 0;
+}
+
+static void
+test_loopbreak(void)
+{
+	struct event ev1, ev2;
+	struct timeval tv;
+
+	setup_test("Loop break: ");
+
+	tv.tv_sec = 0;
+	tv.tv_usec = 0;
+	evtimer_set(&ev1, break_cb, NULL);
+	evtimer_add(&ev1, &tv);
+	evtimer_set(&ev2, fail_cb, NULL);
+	evtimer_add(&ev2, &tv);
+
+	event_dispatch();
+
+	evtimer_del(&ev1);
+	evtimer_del(&ev2);
+
+	cleanup_test();
+}
+
+static void
 test_evbuffer(void) {
 
 	struct evbuffer *evb = evbuffer_new();
@@ -694,12 +823,12 @@ test_evbuffer(void) {
 	cleanup_test();
 }
 
-void
+static void
 test_evbuffer_find(void)
 {
 	u_char* p;
-	char* test1 = "1234567890\r\n";
-	char* test2 = "1234567890\r";
+	const char* test1 = "1234567890\r\n";
+	const char* test2 = "1234567890\r";
 #define EVBUFFER_INITIAL_LENGTH 256
 	char test3[EVBUFFER_INITIAL_LENGTH];
 	unsigned int i;
@@ -749,7 +878,7 @@ test_evbuffer_find(void)
 	evbuffer_free(buf);
 }
 
-void
+static void
 readcb(struct bufferevent *bev, void *arg)
 {
 	if (EVBUFFER_LENGTH(bev->input) == 8333) {
@@ -758,20 +887,20 @@ readcb(struct bufferevent *bev, void *arg)
 	}
 }
 
-void
+static void
 writecb(struct bufferevent *bev, void *arg)
 {
 	if (EVBUFFER_LENGTH(bev->output) == 0)
 		test_ok++;
 }
 
-void
+static void
 errorcb(struct bufferevent *bev, short what, void *arg)
 {
 	test_ok = -2;
 }
 
-void
+static void
 test_bufferevent(void)
 {
 	struct bufferevent *bev1, *bev2;
@@ -807,7 +936,7 @@ struct test_pri_event {
 	int count;
 };
 
-void
+static void
 test_priorities_cb(int fd, short what, void *arg)
 {
 	struct test_pri_event *pri = arg;
@@ -824,7 +953,7 @@ test_priorities_cb(int fd, short what, void *arg)
 	event_add(&pri->ev, &tv);
 }
 
-void
+static void
 test_priorities(int npriorities)
 {
 	char buf[32];
@@ -887,7 +1016,7 @@ test_multiple_cb(int fd, short event, void *arg)
 		test_ok |= 2;
 }
 
-void
+static void
 test_multiple_events_for_same_fd(void)
 {
    struct event e1, e2;
@@ -910,9 +1039,11 @@ test_multiple_events_for_same_fd(void)
    cleanup_test();
 }
 
-int decode_int(uint32_t *pnumber, struct evbuffer *evbuf);
+int evtag_decode_int(uint32_t *pnumber, struct evbuffer *evbuf);
+int evtag_encode_tag(struct evbuffer *evbuf, uint32_t number);
+int evtag_decode_tag(uint32_t *pnumber, struct evbuffer *evbuf);
 
-void
+static void
 read_once_cb(int fd, short event, void *arg)
 {
 	char buf[256];
@@ -931,7 +1062,7 @@ read_once_cb(int fd, short event, void *arg)
 	called++;
 }
 
-void
+static void
 test_want_only_once(void)
 {
 	struct event ev;
@@ -957,7 +1088,7 @@ test_want_only_once(void)
 
 #define TEST_MAX_INT	6
 
-void
+static void
 evtag_int_test(void)
 {
 	struct evbuffer *tmp = evbuffer_new();
@@ -977,7 +1108,7 @@ evtag_int_test(void)
 	}
 
 	for (i = 0; i < TEST_MAX_INT; i++) {
-		if (decode_int(&integer, tmp) == -1) {
+		if (evtag_decode_int(&integer, tmp) == -1) {
 			fprintf(stderr, "decode %d failed", i);
 			exit(1);
 		}
@@ -997,7 +1128,7 @@ evtag_int_test(void)
 	fprintf(stdout, "\t%s: OK\n", __func__);
 }
 
-void
+static void
 evtag_fuzz(void)
 {
 	u_char buffer[4096];
@@ -1040,7 +1171,47 @@ evtag_fuzz(void)
 	fprintf(stdout, "\t%s: OK\n", __func__);
 }
 
-void
+static void
+evtag_tag_encoding(void)
+{
+	struct evbuffer *tmp = evbuffer_new();
+	uint32_t integers[TEST_MAX_INT] = {
+		0xaf0, 0x1000, 0x1, 0xdeadbeef, 0x00, 0xbef000
+	};
+	uint32_t integer;
+	int i;
+
+	for (i = 0; i < TEST_MAX_INT; i++) {
+		int oldlen, newlen;
+		oldlen = EVBUFFER_LENGTH(tmp);
+		evtag_encode_tag(tmp, integers[i]);
+		newlen = EVBUFFER_LENGTH(tmp);
+		fprintf(stdout, "\t\tencoded 0x%08x with %d bytes\n",
+		    integers[i], newlen - oldlen);
+	}
+
+	for (i = 0; i < TEST_MAX_INT; i++) {
+		if (evtag_decode_tag(&integer, tmp) == -1) {
+			fprintf(stderr, "decode %d failed", i);
+			exit(1);
+		}
+		if (integer != integers[i]) {
+			fprintf(stderr, "got %x, wanted %x",
+			    integer, integers[i]);
+			exit(1);
+		}
+	}
+
+	if (EVBUFFER_LENGTH(tmp) != 0) {
+		fprintf(stderr, "trailing data");
+		exit(1);
+	}
+	evbuffer_free(tmp);
+
+	fprintf(stdout, "\t%s: OK\n", __func__);
+}
+
+static void
 evtag_test(void)
 {
 	fprintf(stdout, "Testing Tagging:\n");
@@ -1049,10 +1220,12 @@ evtag_test(void)
 	evtag_int_test();
 	evtag_fuzz();
 
+	evtag_tag_encoding();
+
 	fprintf(stdout, "OK\n");
 }
 
-void
+static void
 rpc_test(void)
 {
 	struct msg *msg, *msg2;
@@ -1060,6 +1233,7 @@ rpc_test(void)
 	struct run *run;
 	struct evbuffer *tmp = evbuffer_new();
 	struct timeval tv_start, tv_end;
+	uint32_t tag;
 	int i;
 
 	fprintf(stdout, "Testing RPC: ");
@@ -1091,10 +1265,20 @@ rpc_test(void)
 		exit(1);
 	}
 
-	evtag_marshal_msg(tmp, 0, msg);
+	evtag_marshal_msg(tmp, 0xdeaf, msg);
+
+	if (evtag_peek(tmp, &tag) == -1) {
+		fprintf(stderr, "Failed to peak tag.\n");
+		exit (1);
+	}
+
+	if (tag != 0xdeaf) {
+		fprintf(stderr, "Got incorrect tag: %0x.\n", tag);
+		exit (1);
+	}
 
 	msg2 = msg_new();
-	if (evtag_unmarshal_msg(tmp, 0, msg2) == -1) {
+	if (evtag_unmarshal_msg(tmp, 0xdeaf, msg2) == -1) {
 		fprintf(stderr, "Failed to unmarshal message.\n");
 		exit(1);
 	}
@@ -1155,12 +1339,18 @@ main (int argc, char **argv)
 
 	test_free_active_base();
 
+	test_event_base_new();
+
 	http_suite();
 
 	rpc_suite();
 
 	dns_suite();
 	
+#ifndef WIN32
+	test_fork();
+#endif
+
 	test_simpleread();
 
 	test_simplewrite();
@@ -1177,6 +1367,7 @@ main (int argc, char **argv)
 	test_immediatesignal();
 #endif
 	test_loopexit();
+	test_loopbreak();
 
 	test_multiple_events_for_same_fd();
 
